import java.awt.Color;
import java.awt.Container;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.geom.Rectangle2D;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SpringLayout;

import com.mindfusion.diagramming.AdjustmentHandles;
import com.mindfusion.diagramming.Diagram;
import com.mindfusion.diagramming.DiagramAdapter;
import com.mindfusion.diagramming.DiagramView;
import com.mindfusion.diagramming.DirectionConstraint;
import com.mindfusion.diagramming.HandlesStyle;
import com.mindfusion.diagramming.NodeEvent;
import com.mindfusion.diagramming.ShadowsStyle;
import com.mindfusion.diagramming.Shape;
import com.mindfusion.diagramming.ShapeNode;
import com.mindfusion.diagramming.SolidBrush;


public class MainFrame extends JFrame
{
	public MainFrame()
	{
		initFrame();
		initDiagram();
	}

	private void initDiagram()
	{
		diagram.setBackBrush(new SolidBrush(Color.white));
		diagram.setShadowsStyle(ShadowsStyle.None);
		diagram.setAlignToGrid(false);

		diagram.addDiagramListener(new DiagramAdapter()
		{
			public void nodeCreated(NodeEvent e) 
			{
				diagramNodeCreated(e);
			}
		});
		
		backbone = diagram.getFactory().createShapeNode(
			0, 60, 300, 1.5d, Shape.fromId("Rectangle"));

		backbone.setBrush(new SolidBrush(Color.black));
		backbone.setEnabledHandles(AdjustmentHandles.None);
		backbone.setHandlesStyle(HandlesStyle.Invisible);

		diagramView.setCustomBehavior(
			new FishboneBehavior(diagramView, backbone));
	}

	private void diagramNodeCreated(NodeEvent e)
	{
		if (e.getNode() instanceof CauseNode) 
			e.getNode().getConstraints().setMoveDirection(DirectionConstraint.Horizontal);
	}

	private void initFrame()
	{
		diagram = new Diagram();
		diagram.setBounds(new Rectangle2D.Float(0, 0, 210, 297));
		diagramView = new DiagramView(diagram);
		scrP = new JScrollPane(diagramView);
		tArInfo = new JTextArea();
		tArInfo.setWrapStyleWord(true);
		tArInfo.setEditable(false);
		tArInfo.setLineWrap(true);

		tArInfo.setText(
			"This sample lets you create Fishbone diagrams interactively. It demonstrates " +
			"how to create derived node classes and implement custom drawing by overriding " +
			"the draw and drawShadow methods, custom interaction handling by overriding " +
			"startCreate, updateCreate and completeCreate, and custom hit-testing by overriding " +
			"containsPoint. To try this, create cause nodes by drawing from the line above , " +
			"and then create aspect nodes by drawing from the cause nodes.");

		Container cp = getContentPane();
		cp.add(scrP);
		cp.add(tArInfo);
		
		javax.swing.SpringLayout layout = new javax.swing.SpringLayout();
		layout.putConstraint(SpringLayout.SOUTH, tArInfo, 0, SpringLayout.SOUTH, cp);
		layout.putConstraint(SpringLayout.EAST, tArInfo, 0, SpringLayout.EAST, cp);
		layout.putConstraint(SpringLayout.WEST, tArInfo, 0, SpringLayout.WEST, cp);
		layout.putConstraint(SpringLayout.NORTH, tArInfo, -80, SpringLayout.SOUTH, cp);
		
		layout.putConstraint(SpringLayout.EAST, scrP, 0, SpringLayout.EAST, cp);
		layout.putConstraint(SpringLayout.SOUTH, scrP, 0, SpringLayout.NORTH, tArInfo);
		layout.putConstraint(SpringLayout.NORTH, scrP, 0, SpringLayout.NORTH, cp);
		layout.putConstraint(SpringLayout.WEST, scrP, 0, SpringLayout.WEST, cp);
				
		cp.setLayout(layout);
		
		setTitle("MindFusion.Diagramming Sample: Fishbone Diagram");
		setSize(736, 586);
		
		addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				System.exit(0);
			}
		});
	}

	public static Rectangle2D fromLTRB(double l, double t, double r, double b)
	{
		return new Rectangle2D.Double(l, t, r - l, b - t);
	}
	
	private Diagram diagram;
	ShapeNode backbone;
	private DiagramView diagramView;
	
	private JTextArea tArInfo;
	private JScrollPane scrP;
	
	private static final long serialVersionUID = 1L;
}