import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

import javax.swing.*;

import com.mindfusion.diagramming.*;

public class InteractiveTableFrame extends JFrame
{
	public InteractiveTableFrame()
	{
		super("MindFusion.Diagramming sample: Interactive Table");
		initFrame();
		initDiagram();
	}

	private void initDiagram()
	{
        diagram.setCellFrameStyle(CellFrameStyle.Simple);
        diagram.setTableHandlesStyle(HandlesStyle.HatchHandles3);
		diagram.setTableConnectionStyle(TableConnectionStyle.Table);
		diagram.setTableColumnCount(1);
        diagram.setLinkHeadShapeSize(15);
        diagram.setTableRowCount(4);
        diagram.setTablesScrollable(true);
        diagram.setTableCaptionHeight(15);
        diagram.setTableRowHeight(18);
        diagram.setAdjustmentHandlesSize(6);
        diagram.setBackBrush(new SolidBrush(new Color((int)242, (int)255, (int)216)));
        diagram.setMeasureUnit(GraphicsUnit.Point);
        diagram.setTablesScrollable(false);
        diagram.setSelectAfterCreate(true);
        
        TableNodeStyle tableStyle = diagram.getTableNodeStyle();
        tableStyle.setStroke(new SolidBrush(Color.black));
        tableStyle.setStrokeThickness(0);
        tableStyle.setBrush(new SolidBrush(Color.white));
        
        DiagramStyle nodesStyle = diagram.getDiagramStyle();
        nodesStyle.setFontFamily("Arial");
        nodesStyle.setFontSize(9);
        nodesStyle.setBrush(new SolidBrush(Color.LIGHT_GRAY));
        
        diagram.getNodeEffects().add(new GlassEffect());

        diagram.addDiagramListener(new DiagramAdapter()
        {
        	public void nodeCreated(NodeEvent e) 
        	{
        		if (e.getNode() instanceof TableNode)
        			diag_TableCreated();
        	}

        	public void nodeDeleted(NodeEvent e) 
        	{
        		if(e.getNode() instanceof TableNode)
        			TableDeleted();
        	}

        	public void cellClicked(CellEvent e) {
    			if(e.getMouseButton() == MouseEvent.BUTTON1)
    			{
    				onCellClicked(e);
    			}
        	}
        });

        diagramView.setBehavior(Behavior.LinkTables);

        String infoString =
			"This sample shows how to use a combo box with values available for a table's cell. " +
			"Draw a table here and click on its cells to display the combo box.";
        ShapeNode _box = diagram.getFactory().createShapeNode(5,5, 170, 60);
        _box.setText(infoString);
        _box.setTransparent(true);
        _box.setLocked(true);
	}


    /**
     * Shutdown procedure when run as an application.
     */
    protected void windowClosed()
    {

    	// TODO: Check if it is safe to close the application

        // Exit application.
        System.exit(0);
    }


    private void diag_TableCreated()
    {
    	Reset();
    }

	public void onCellClicked(CellEvent e)
	{
		_zoomFactor = diagramView.getZoomFactor();

		_table = e.getTable();
		Rectangle2D.Float r = _table.getBounds();

		_comboBox.repaint();

		Point2D.Float docPt = e.getMousePosition();
		Point2D.Float devPt = docToDevice(docPt);
		int x = (int)devPt.getX();
		int y = (int)devPt.getY();


		Point2D.Float tblDocPos = new Point2D.Float((float)r.getX(), (float)r.getY());
		Point2D.Float tblDevPos = docToDevice(tblDocPos);

		Point2D.Float cellDocSize = new Point2D.Float(
									(float)_table.getBounds().getWidth(), (float)_table.getRowHeight());
		Point2D.Float cellDevSize = docToDevice(cellDocSize);

		Point2D.Float capDocSize = new Point2D.Float(0, _table.getCaptionHeight());
		Point2D.Float capDevSize = docToDevice(capDocSize);

		_row = e.getRow();

		float m =(float) r.getY() + (float)e.getTable().getCaptionHeight() +
					 	(float)e.getTable().getRowHeight()*_row + 2;
		Point2D.Float posDocY = new Point2D.Float(0, m);
		Point2D.Float posDevY = docToDevice(posDocY);

		_comboBox.setBounds((int)tblDevPos.getX(), (int)posDevY.getY(),
										(int)(cellDevSize.getX() - diagramView.getX()),
										(int)(cellDevSize.getY() - diagramView.getY()));

		for(int i = 0; i < _comboBox.getItemCount(); i++)
		{
			if(_comboBox.getItemAt(i).toString().compareTo(e.getCell().getText()) == 0)
			{
				_comboBox.setSelectedIndex(i);
				break;
			}

			if(i == (_comboBox.getItemCount() - 1))
			{
				_comboBox.setSelectedIndex(0);
			}
		}
		_comboBox.repaint();
	}

	private void TableDeleted()
	{
		Reset();
	}

	private void _comboBox_Leave(ActionEvent e)
	{
		_comboBox.repaint();
		String text = _comboBox.getSelectedItem().toString();
		_table.getCell(0, _row).setText(text);
	}


	private int _row = -1;
	private int _col = -1;
	private TableNode _table = null;

	private void Reset()
	{
		_comboBox.setBounds(0, 1010, 0, 0);
		_row = -1;
		_col = -1;
		_table = null;
	}

	AffineTransform getScreenTransform()
    {
    	GraphicsConfiguration gc = getGraphicsConfiguration();
    	if (gc != null)
    	{
	        AffineTransform at = (AffineTransform)
	        	gc.getDefaultTransform().clone();

			double dpi = Toolkit.getDefaultToolkit().getScreenResolution();
			at.scale(dpi / 72, dpi / 72);

	        at.scale(_zoomFactor / 100.0f, _zoomFactor / 100.0f);
			at.translate(- _docBounds.x -diagramView.getScrollX(), - _docBounds.y -diagramView.getScrollY());

	        return at;
		}

		return null;
    }

    Point2D.Float docToDevice(Point2D.Float point)
	{
		Graphics2D g = (Graphics2D)getGraphics();
		AffineTransform transform = getScreenTransform();

        // get the corner points
        Point2D.Float dst = new Point2D.Float();

        // get the world coordinates
        transform.transform(point, dst);

        // return rectangle in world coordinates
        return dst;
	}
    
	private void initFrame()
	{
		Container cp;
    	JScrollPane _scrollPane;
		JPanel _flowPanel ;
		
        String[] animals =
        {
	        	" ",
	            "Bear",
	            "Eagle",
	            "Elephant",
	            "Mouse",
	            "Whale"
        	};
        _comboBox = new JComboBox(animals);

        diagram = new Diagram();
        diagramView = new DiagramView(diagram);
        _scrollPane = new JScrollPane(diagramView);
		_docBounds = new Rectangle2D.Float(0, 0, 1000, 1000);
        //
        //_scroll
        //
        _scrollPane.setName("_scrPane");
		_scrollPane.setAutoscrolls(true);        

        //
        //_comboBox
        //
        _comboBox.setName("_comboBox");
        _comboBox.setBackground(Color.white);
        _comboBox.setEditable(false);
        _comboBox.setVisible(true);
        _comboBox.setSelectedIndex(0);
      	_comboBox.setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
        _comboBox.setOpaque(true);
        _comboBox.setBounds(1100, 1100, 40, 25);
        _comboBox.addActionListener(new ActionListener(){
        	public void actionPerformed(ActionEvent e){
        		_comboBox_Leave(e);
        	}
        });

        _flowPanel = new JPanel();
        _flowPanel.setLayout(new BoxLayout(_flowPanel, BoxLayout.Y_AXIS));
        _flowPanel.add(_scrollPane);
        _flowPanel.setBounds(20,20, 200, 200);

        cp = this.getContentPane();
        cp.add(_comboBox);
        cp.add(_flowPanel);

        setSize(new Dimension(400, 400));

        // Add window listener.
        this.addWindowListener
        (
            new WindowAdapter()
            {
                public void windowClosing(WindowEvent e)
                {
                    InteractiveTableFrame.this.windowClosed();
                }
            }
        );
	}

    private Diagram diagram;
    private DiagramView diagramView;
    private JComboBox _comboBox;
    private float _zoomFactor = 0;
    private Rectangle2D.Float _docBounds;

    private static final long serialVersionUID = 1L;
}