package com.mindfusion.diagramming.samples.demo;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;

import javax.swing.*;
import javax.swing.border.LineBorder;
import javax.swing.plaf.synth.ColorType;

public class AddTreeViewItemDialog extends JDialog
{
	public AddTreeViewItemDialog(JFrame parent)
	{
		super(parent, "Add TreeViewNode", true);
		_accepted = false;
		
		lbLabel = new JLabel("Label");
		lbLabel.setBounds(10, 10, 50, 25);
		
		tFlLabel = new JTextField();
		tFlLabel.setBounds(65, 10, 170, 25);
		
		lbIcon = new JLabel("Icon");
		lbIcon.setBounds(10, 40, 50, 25);
		
		ListRenderer listrender = new ListRenderer(4);
		listrender.createBufferedImage(0, "1", getIcon("Files/_0.png"));
		listrender.createBufferedImage(1, "2", getIcon("Files/_1.png"));
		listrender.createBufferedImage(2, "3", getIcon("Files/_2.png"));
		listrender.createBufferedImage(3, "4", getIcon("Files/_3.png"));
		DefaultListModel _modelN = new DefaultListModel();
		_modelN.addElement(0);
		_modelN.addElement(1);
		_modelN.addElement(2);
		_modelN.addElement(3);

		lstIcons = new JList();
		lstIcons.setDoubleBuffered(true);
		lstIcons.setModel(_modelN);
		lstIcons.setCellRenderer(listrender);
		lstIcons.setBounds(65, 40, 170, 110);

		lbColor = new JLabel("Color");
		lbColor.setBounds(10, 165, 50, 25);
		
		pnlColorView = new JPanel();
		pnlColorView.setBackground(Color.black);
		pnlColorView.setBounds(65, 165, 60, 25);
		pnlColorView.addMouseListener(new MouseAdapter(){
			public void mouseClicked(java.awt.event.MouseEvent arg0)
			{
				onColorViewClick();
			}
		});
		
		btnAdd = new JButton("Add");
		btnAdd.setBounds(100, 210, 60, 30);
		btnAdd.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0)
			{
				_accepted = true;
				AddTreeViewItemDialog.this.setVisible(false);
			}
		});
		
		this.setLayout(null);
		this.add(lbLabel);
		this.add(tFlLabel);
		this.add(lbIcon);
		this.add(lstIcons);
		this.add(lbColor);
		this.add(pnlColorView);
		this.add(btnAdd);
		
		this.setSize(260, 300);
		this.setResizable(false);
		this.setAlwaysOnTop(true);

		// Center in parent
		Rectangle parentBounds = new Rectangle();
		parent.getBounds(parentBounds);

		setLocation((parentBounds.width - getWidth()) / 2,
			(parentBounds.height - getHeight()) / 2);

	}
	
	/**
	 * Gets the image with the specified name.
	 */
	private ImageIcon getIcon(String fileName)
	{
		java.net.URL imageURL = AddTreeViewItemDialog.class.getResource(fileName);
		if (imageURL != null)
		    return new ImageIcon(imageURL);

		return null;
	}

	private void onColorViewClick()
	{
		Color newColor = JColorChooser.showDialog(this, 
				"Choose color", pnlColorView.getBackground());
		
		if (newColor != null)
			pnlColorView.setBackground(newColor);
	}
	
	public int getItemImageIndex()
	{
		return -1;
	}
	
	public Image getItemImage()
	{
		ListRenderer listrender = (ListRenderer)lstIcons.getCellRenderer();
		int index = lstIcons.getSelectedIndex();
		if (index == -1)
			return null;
		return listrender.listImage[index].getImage();
	}
	
	public Color getItemColor()
	{
		return pnlColorView.getBackground();
	}
	
	public String getItemLabel()
	{
		return tFlLabel.getText();
	}
	
	/**
	 * Gets whether the user has confirmed the dialog.
	 */
	public boolean getAccepted() { return _accepted; }
	
	private boolean _accepted;
	
	private JLabel lbLabel;
	private JTextField tFlLabel;
	private JLabel lbIcon;
	private JList lstIcons;
	private JLabel lbColor;
	private JPanel pnlColorView;
	private JButton btnAdd;
}

class ListRenderer extends JLabel
	implements ListCellRenderer {

	String[] textImage;
	ImageIcon[] listImage;
	
	public ListRenderer(int size)
	{
		this.textImage = new String[size];
		this.listImage = new ImageIcon[size];
	}
	
	public void createBufferedImage( int index, String textImage, ImageIcon bufferedImage)
	{
		this.textImage[index] = textImage;
		this.listImage[index] = bufferedImage;
	}
	
	/**
	* This method finds the image and text corresponding
	* to the selected value and returns the label, set up
	* to display the text and image.
	*/
	public Component getListCellRendererComponent(JList list, Object value,
							int index, boolean isSelected, boolean cellHasFocus)
	{
	
		//Get the selected index. (The index param isn't
		//always valid, so just use the value.)
		
		if (isSelected)
		{
			setBackground(list.getSelectionBackground());
			setForeground(list.getSelectionForeground());
		}
		else
		{
			setBackground(list.getBackground());
			setForeground(list.getForeground());
		}
		
		if (isSelected)
			setBorder(new LineBorder(new Color(0,0,255)));
		else
			setBorder(new LineBorder(Color.white));
		
		//Set the icon and text.  If icon was null, say so.
		setText(textImage[index]);
		setIcon(listImage[index]);
		return this;
	}
}
