/**
 * Copyright (c) 2013, MindFusion LLC - Bulgaria.
 */

package samples.magnifier;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.imageio.*;
import javax.swing.*;
import javax.swing.event.*;

import com.mindfusion.diagramming.*;
import com.mindfusion.diagramming.Shape;


public class MainFrame extends JFrame
	implements ChangeListener, ActionListener, ShapeListBoxListener
{
	public MainFrame()
	{
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setLocationByPlatform(true);
		setSize(800, 600);
		setTitle("JDiagram Sample: Magnifier");
		try
		{
			ArrayList<Image> images = new ArrayList<Image>();
			images.add(ImageIO.read(App.class.getResourceAsStream("App.png")));
			images.add(ImageIO.read(App.class.getResourceAsStream("App32.png")));
			setIconImages(images);
		}
		catch (Exception ex)
		{
		}

		initializeComponent();
	}

	private void initializeComponent()
	{
		setLayout(new BorderLayout());

		panel1 = new JPanel(new VerticalFlowLayout(VerticalFlowLayout.LEFT, VerticalFlowLayout.TOP, 5, 0));
		groupBoxEnableMagnifier = new JPanel(new VerticalFlowLayout(VerticalFlowLayout.LEFT, VerticalFlowLayout.TOP, 0, 0));
		radioNever = new JRadioButton("Never");
		radioOnMouseClick = new JRadioButton("On mouse click");
		radioAlways = new JRadioButton("Always");
		groupEnableMagnifier = new ButtonGroup();
		labelSize = new JLabel("Size:");
		trackBarSize = new JSlider();
		labelZoom = new JLabel("Zoom Factor:");
		trackBarZoom = new JSlider();
		labelShape = new JLabel("Shape:");
		shapeListBox = new ShapeListBox();
		shapeListBoxPane = new JScrollPane(shapeListBox);
		diagramView = new DiagramView();
		scrollPane = new JScrollPane(diagramView);
		// panel1
		panel1.setPreferredSize(new Dimension(200, 200));
		panel1.add(Box.createVerticalStrut(10));
		panel1.add(groupBoxEnableMagnifier, Integer.valueOf(VerticalFlowLayout.STRETCH));
		panel1.add(Box.createVerticalStrut(5));
		panel1.add(labelSize);
		panel1.add(Box.createVerticalStrut(2));
		panel1.add(trackBarSize, Integer.valueOf(VerticalFlowLayout.STRETCH));
		panel1.add(Box.createVerticalStrut(5));
		panel1.add(labelZoom);
		panel1.add(Box.createVerticalStrut(5));
		panel1.add(trackBarZoom, Integer.valueOf(VerticalFlowLayout.STRETCH));
		panel1.add(Box.createVerticalStrut(5));
		panel1.add(labelShape);
		panel1.add(Box.createVerticalStrut(5));
		panel1.add(shapeListBoxPane);
		// groupBoxEnableMagnifier
		groupBoxEnableMagnifier.setBorder(BorderFactory.createTitledBorder("Enable Magnifier"));
		// radioNever
		radioNever.setSelected(true);
		radioNever.addActionListener(this);
		// radioOnMouseClick
		radioOnMouseClick.addActionListener(this);
		// radioAlways
		radioAlways.addActionListener(this);
		// groupBoxEnableMagnifier
		groupBoxEnableMagnifier.add(radioNever);
		groupBoxEnableMagnifier.add(radioOnMouseClick);
		groupBoxEnableMagnifier.add(radioAlways);
		// groupEnableMagnifier
		groupEnableMagnifier.add(radioNever);
		groupEnableMagnifier.add(radioOnMouseClick);
		groupEnableMagnifier.add(radioAlways);
		// trackBarSize
		trackBarSize.setMinimum(10);
		trackBarSize.setMaximum(200);
		trackBarSize.setValue(80);
		trackBarSize.addChangeListener(this);
		// trackBarZoom
		trackBarZoom.setMinimum(10);
		trackBarZoom.setMaximum(510);
		trackBarZoom.setValue(150);
		trackBarZoom.addChangeListener(this);
		// shapeListBox
		shapeListBox.setDragEnabled(false);
		shapeListBox.addShapeListener(this);
		// shapeListBoxPane
		shapeListBoxPane.setPreferredSize(new Dimension(190, 300));
		shapeListBoxPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

		// Structure
		Container c = getContentPane();
		c.add(panel1, BorderLayout.EAST);
		c.add(scrollPane, BorderLayout.CENTER);
	}

	/**
	 * {@link ActionLisener#actionPerformed} implementation.
	 */
	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource() == radioNever && radioNever.isSelected())
		{
			diagramView.setMagnifierEnabled(false);
			diagramView.setBehavior(Behavior.LinkShapes);
		}
		else if (e.getSource() == radioOnMouseClick && radioOnMouseClick.isSelected())
		{
			diagramView.setMagnifierEnabled(false);
			diagramView.setBehavior(Behavior.Magnify);
		}
		else if (e.getSource() == radioAlways && radioAlways.isSelected())
		{
			diagramView.setMagnifierEnabled(true);
			diagramView.setBehavior(Behavior.LinkShapes);
		}
	}

	/**
	 * {@link ChangeListener#stateChanged} implementation.
	 */
	public void stateChanged(ChangeEvent e)
	{
		if (e.getSource() == trackBarSize)
		{
			diagramView.setMagnifierWidth(trackBarSize.getValue() * 2);
			diagramView.setMagnifierHeight(trackBarSize.getValue() * 2);
		}
		else if (e.getSource() == trackBarZoom)
		{
			diagramView.setMagnifierFactor(trackBarZoom.getValue());
		}
	}

	/**
	 * {@link ShapeListBoxListener#shapeSelected} implementation.
	 */
	public void shapeSelected(Shape shape, int index)
	{
		diagramView.setMagnifierShape(shape);
	}

	/**
	 * {@link ShapeListBoxListener#shapeRemoved} implementation.
	 */
	public void shapeRemoved(Shape shape, int oldIndex)
	{
	}

	/**
	 * {@link ShapeListBoxListener#shapeRenamed} implementation.
	 */
	public void shapeRenamed(Shape shape, int index, String oldName)
	{
	}


	private static final long serialVersionUID = 1L;

	private JPanel panel1;
	private DiagramView diagramView;
	private JScrollPane scrollPane;
	private JPanel groupBoxEnableMagnifier;
	private JRadioButton radioNever;
	private JRadioButton radioOnMouseClick;
	private JRadioButton radioAlways;
	private ButtonGroup groupEnableMagnifier;
	private JLabel labelSize;
	private JSlider trackBarSize;
	private JLabel labelZoom;
	private JSlider trackBarZoom;
	private JLabel labelShape;
	private ShapeListBox shapeListBox;
	private JScrollPane shapeListBoxPane;
}
